@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -140,18 +140,95 @@ class CHistoryManager {
 	private function getLastValuesFromSql($items, $limit, $period) {
 		$results = [];
 
-		foreach ($items as $item) {
-			$values = DBfetchArray(DBselect(
-				'SELECT *'.
-				' FROM '.self::getTableName($item['value_type']).' h'.
-				' WHERE h.itemid='.zbx_dbstr($item['itemid']).
-					($period ? ' AND h.clock>'.(time() - $period) : '').
-				' ORDER BY h.clock DESC',
-				$limit
-			));
+		if ($period) {
+			$period = time() - $period;
+		}
 
-			if ($values) {
-				$results[$item['itemid']] = $values;
+		if ($limit == 1) {
+			foreach ($items as $item) {
+				// Executing two subsequent queries individually for the sake of performance.
+
+				$clock_max = DBfetch(DBselect(
+					'SELECT MAX(h.clock)'.
+					' FROM '.self::getTableName($item['value_type']).' h'.
+					' WHERE h.itemid='.zbx_dbstr($item['itemid']).
+						($period ? ' AND h.clock>'.$period : '')
+				), false);
+
+				if ($clock_max) {
+					$clock_max = reset($clock_max);
+
+					if ($clock_max !== null) {
+						$values = DBfetchArray(DBselect(
+							'SELECT *'.
+							' FROM '.self::getTableName($item['value_type']).' h'.
+							' WHERE h.itemid='.zbx_dbstr($item['itemid']).
+								' AND h.clock='.zbx_dbstr($clock_max).
+							' ORDER BY h.ns DESC',
+							$limit
+						));
+
+						if ($values) {
+							$results[$item['itemid']] = $values;
+						}
+					}
+				}
+			}
+		}
+		else {
+			foreach ($items as $item) {
+				// Cannot order by h.ns directly here due to performance issues.
+				$values = DBfetchArray(DBselect(
+					'SELECT *'.
+					' FROM '.self::getTableName($item['value_type']).' h'.
+					' WHERE h.itemid='.zbx_dbstr($item['itemid']).
+						($period ? ' AND h.clock>'.$period : '').
+					' ORDER BY h.clock DESC',
+					$limit + 1
+				));
+
+				if ($values) {
+					$count = count($values);
+					$clock = $values[$count - 1]['clock'];
+
+					if ($count == $limit + 1 && $values[$count - 2]['clock'] == $clock) {
+						/*
+						 * The last selected entries having the same clock means the selection (not just the order)
+						 * of the last entries is possibly wrong due to unordered by nanoseconds.
+						 */
+
+						do {
+							unset($values[--$count]);
+						} while ($values && $values[$count - 1]['clock'] == $clock);
+
+						$db_values = DBselect(
+							'SELECT *'.
+							' FROM '.self::getTableName($item['value_type']).' h'.
+							' WHERE h.itemid='.zbx_dbstr($item['itemid']).
+								' AND h.clock='.$clock.
+							' ORDER BY h.ns DESC',
+							$limit - $count
+						);
+
+						while ($db_value = DBfetch($db_values)) {
+							$values[] = $db_value;
+							$count++;
+						}
+					}
+
+					CArrayHelper::sort($values, [
+						['field' => 'clock', 'order' => ZBX_SORT_DOWN],
+						['field' => 'ns', 'order' => ZBX_SORT_DOWN]
+					]);
+
+					$values = array_values($values);
+
+					while ($count > $limit) {
+						unset($values[--$count]);
+					}
+
+					$results[$item['itemid']] = $values;
+				}
 			}
 		}
 
@@ -164,13 +241,15 @@ class CHistoryManager {
 	 *
 	 * The $item parameter must have the value_type and itemid properties set.
 	 *
-	 * @param array $item     item to get value for
-	 * @param int   $clock    timestamp (seconds)
-	 * @param int   $ns       timestamp (nanoseconds)
+	 * @param array  $item
+	 * @param string $item['itemid']
+	 * @param int    $item['value_type']
+	 * @param int    $clock
+	 * @param int    $ns
 	 *
-	 * @return string    value at specified time of first value before specified time
+	 * @return string|null  Value at specified time of first value before specified time. null if value is not found.
 	 */
-	public function getValueAt($item, $clock, $ns) {
+	public function getValueAt(array $item, $clock, $ns) {
 		switch (self::getDataSourceType($item['value_type'])) {
 			case ZBX_HISTORY_SOURCE_ELASTIC:
 				return $this->getValueAtFromElasticsearch($item, $clock, $ns);
@@ -185,7 +264,7 @@ class CHistoryManager {
 	 *
 	 * @see CHistoryManager::getValueAt
 	 */
-	private function getValueAtFromElasticsearch($item, $clock, $ns) {
+	private function getValueAtFromElasticsearch(array $item, $clock, $ns) {
 		$query = [
 			'sort' => [
 				'clock' => ZBX_SORT_DOWN,
@@ -224,7 +303,7 @@ class CHistoryManager {
 					'range' => [
 						'clock' => [
 							'lt' => $clock
-						]
+						] + (ZBX_HISTORY_PERIOD ? ['gte' => $clock - ZBX_HISTORY_PERIOD] : [])
 					]
 				]
 			]
@@ -253,7 +332,7 @@ class CHistoryManager {
 	 *
 	 * @see CHistoryManager::getValueAt
 	 */
-	private function getValueAtFromSql($item, $clock, $ns) {
+	private function getValueAtFromSql(array $item, $clock, $ns) {
 		$value = null;
 		$table = self::getTableName($item['value_type']);
 
@@ -286,7 +365,8 @@ class CHistoryManager {
 			$sql = 'SELECT MAX(clock) AS clock'.
 					' FROM '.$table.
 					' WHERE itemid='.zbx_dbstr($item['itemid']).
-						' AND clock<'.zbx_dbstr($clock);
+						' AND clock<'.zbx_dbstr($clock).
+						(ZBX_HISTORY_PERIOD ? ' AND clock>='.zbx_dbstr($clock - ZBX_HISTORY_PERIOD) : '');
 
 			if (($row = DBfetch(DBselect($sql))) !== false) {
 				$max_clock = $row['clock'];
@@ -546,10 +626,6 @@ class CHistoryManager {
 				$sql_from = ($item['value_type'] == ITEM_VALUE_TYPE_UINT64) ? 'history_uint' : 'history';
 			}
 			else {
-				if (!$item['has_scheduling_intervals'] || $item['delay'] != 0) {
-					$item['delay'] = max($item['delay'], SEC_PER_HOUR);
-				}
-
 				$sql_select = 'SUM(num) AS count,AVG(value_avg) AS avg,MIN(value_min) AS min,MAX(value_max) AS max';
 				$sql_from = ($item['value_type'] == ITEM_VALUE_TYPE_UINT64) ? 'trends_uint' : 'trends';
 			}
@@ -666,177 +742,6 @@ class CHistoryManager {
 		return null;
 	}
 
-	/**
-	 * Return the time of the 1st appearance of items in history or trends.
-	 *
-	 * @param array $items
-	 * @param array $items[]['itemid']
-	 * @param array $items[]['hostid']
-	 * @param array $items[]['value_type']
-	 * @param array $items[]['history']
-	 * @param array $items[]['trends']
-	 *
-	 * @return int    timestamp (unixtime)
-	 */
-	public function getMinClock(array $items) {
-		$item_types = [
-			ITEM_VALUE_TYPE_FLOAT => [],
-			ITEM_VALUE_TYPE_STR => [],
-			ITEM_VALUE_TYPE_LOG => [],
-			ITEM_VALUE_TYPE_UINT64 => [],
-			ITEM_VALUE_TYPE_TEXT => []
-		];
-		$max = ['history' => 0, 'trends' => 0];
-		$simple_interval_parser = new CSimpleIntervalParser();
-
-		$items = CMacrosResolverHelper::resolveTimeUnitMacros($items, ['history', 'trends']);
-
-		foreach ($items as $item) {
-			$item_types[$item['value_type']][] = $item['itemid'];
-
-			if ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT || $item['value_type'] == ITEM_VALUE_TYPE_UINT64) {
-				if ($simple_interval_parser->parse($item['history']) == CParser::PARSE_SUCCESS) {
-					$item['history'] = (int) timeUnitToSeconds($item['history']);
-					$max['history'] = max($max['history'], $item['history']);
-				}
-
-				if ($simple_interval_parser->parse($item['trends']) == CParser::PARSE_SUCCESS) {
-					$item['trends'] = (int) timeUnitToSeconds($item['trends']);
-					$max['trends'] = max($max['trends'], $item['trends']);
-				}
-			}
-		}
-
-		$source = ($max['history'] > $max['trends']) ? 'history' : 'trends';
-		$storage_items = [];
-
-		foreach ($item_types as $type => $itemids) {
-			if (!$itemids) {
-				continue;
-			}
-
-			$history_source = self::getDataSourceType($type);
-			$storage_items[$history_source][$type] = $itemids;
-		}
-
-		$min_clock = [];
-
-		if (array_key_exists(ZBX_HISTORY_SOURCE_ELASTIC, $storage_items)) {
-			$min_clock[] = $this->getMinClockFromElasticsearch($storage_items[ZBX_HISTORY_SOURCE_ELASTIC]);
-		}
-
-		if (array_key_exists(ZBX_HISTORY_SOURCE_SQL, $storage_items)) {
-			$min_clock[] = $this->getMinClockFromSql($storage_items[ZBX_HISTORY_SOURCE_SQL], $source);
-		}
-
-		$min_clock = min($min_clock);
-
-		// If DB clock column is corrupted having negative numbers, return min clock from max possible history storage.
-		if ($min_clock == 0) {
-			if ($item_types[ITEM_VALUE_TYPE_FLOAT] || $item_types[ITEM_VALUE_TYPE_UINT64]) {
-				$min_clock = time() - max($max['history'], $max['trends']);
-
-				/*
-				 * In case history storage exceeds the maximum time difference between current year and minimum 1970
-				 * (for example year 2014 - 200 years < year 1970), correct year to 1970 (unix time timestamp 0).
-				 */
-				if ($min_clock < 0) {
-					$min_clock = 0;
-				}
-			}
-			else {
-				$min_clock = time() - SEC_PER_YEAR;
-			}
-		}
-
-		return $min_clock;
-	}
-
-	/**
-	 * Elasticsearch specific implementation of getMinClock.
-	 *
-	 * @see CHistoryManager::getMinClock
-	 */
-	private function getMinClockFromElasticsearch(array $items) {
-		$query = [
-			'aggs'=> [
-				'min_clock' => [
-					'min' => [
-						'field' => 'clock'
-					]
-				]
-			],
-			'size'=> 0
-		];
-
-		$min_clock = [];
-
-		foreach (self::getElasticsearchEndpoints(array_keys($items)) as $type => $endpoint) {
-			$query['query']['terms']['itemid'] = $items[$type];
-			$data = CElasticsearchHelper::query('POST', $endpoint, $query);
-
-			// Field value_as_string is used as a workaround for date aggregation being presented as milliseconds.
-			if (array_key_exists('min_clock', $data) && array_key_exists('value_as_string', $data['min_clock'])) {
-				$min_clock[] = $data['min_clock']['value_as_string'];
-			}
-		}
-
-		if ($min_clock) {
-			return min($min_clock);
-		}
-
-		return null;
-	}
-
-	/**
-	 * SQL specific implementation of getMinClock.
-	 *
-	 * @see CHistoryManager::getMinClock
-	 */
-	private function getMinClockFromSql(array $items, $source) {
-		$sql_unions = [];
-
-		foreach ($items as $type => $itemids) {
-			if (!$itemids) {
-				continue;
-			}
-
-			switch ($type) {
-				case ITEM_VALUE_TYPE_FLOAT:
-					$sql_from = $source;
-					break;
-				case ITEM_VALUE_TYPE_STR:
-					$sql_from = 'history_str';
-					break;
-				case ITEM_VALUE_TYPE_LOG:
-					$sql_from = 'history_log';
-					break;
-				case ITEM_VALUE_TYPE_UINT64:
-					$sql_from = $source.'_uint';
-					break;
-				case ITEM_VALUE_TYPE_TEXT:
-					$sql_from = 'history_text';
-					break;
-				default:
-					$sql_from = 'history';
-			}
-
-			foreach ($itemids as $itemid) {
-				$sql_unions[] =
-					'SELECT MIN(h.clock) AS clock'.
-					' FROM '.$sql_from.' h'.
-					' WHERE h.itemid='.zbx_dbstr($itemid);
-			}
-		}
-
-		$row = DBfetch(DBselect(
-			'SELECT MIN(h.clock) AS min_clock'.
-			' FROM ('.implode(' UNION ALL ', $sql_unions).') h'
-		));
-
-		return $row['min_clock'];
-	}
-
 	/**
 	 * Clear item history and trends by provided item IDs. History is deleted from both SQL and Elasticsearch.
 	 *
@@ -1035,7 +940,7 @@ class CHistoryManager {
 
 		foreach ($indices as $type => $index) {
 			if (($url = self::getElasticsearchUrl($index)) !== null) {
-				$endponts[$type] = $url.$index.'/values/'.$action;
+				$endponts[$type] = $url.$index.'*/values/'.$action;
 			}
 		}
 
