@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -52,7 +52,13 @@ class CConfiguration extends CApiService {
 		$writer->formatOutput(false);
 		$export->setWriter($writer);
 
-		return $export->export();
+		$export_data = $export->export();
+
+		if ($export_data === false) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
+		}
+
+		return $export_data;
 	}
 
 	/**
@@ -66,68 +72,68 @@ class CConfiguration extends CApiService {
 			'source' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED],
 			'rules' =>				['type' => API_OBJECT, 'flags' => API_REQUIRED, 'fields' => [
 				'applications' =>		['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'deleteMissing' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'deleteMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'discoveryRules' =>		['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN],
-					'deleteMissing' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false],
+					'deleteMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'graphs' =>				['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN],
-					'deleteMissing' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false],
+					'deleteMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'groups' =>				['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'hosts' =>				['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'httptests' =>			['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN],
-					'deleteMissing' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false],
+					'deleteMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'images' =>				['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'items' =>				['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN],
-					'deleteMissing' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false],
+					'deleteMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'maps' =>				['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'screens' =>			['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'templateLinkage' =>	['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'templates' =>			['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'templateScreens' =>	['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN],
-					'deleteMissing' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false],
+					'deleteMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'triggers' =>			['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN],
-					'deleteMissing' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false],
+					'deleteMissing' =>		['type' => API_BOOLEAN, 'default' => false]
 				]],
 				'valueMaps' =>			['type' => API_OBJECT, 'fields' => [
-					'createMissing' =>		['type' => API_BOOLEAN],
-					'updateExisting' =>		['type' => API_BOOLEAN]
+					'createMissing' =>		['type' => API_BOOLEAN, 'default' => false],
+					'updateExisting' =>		['type' => API_BOOLEAN, 'default' => false]
 				]]
 			]]
 		]];
@@ -147,6 +153,8 @@ class CConfiguration extends CApiService {
 		$converterChain->addConverter('2.0', $importConverterFactory->getObject('2.0'));
 		$converterChain->addConverter('3.0', $importConverterFactory->getObject('3.0'));
 		$converterChain->addConverter('3.2', $importConverterFactory->getObject('3.2'));
+		$converterChain->addConverter('3.4', $importConverterFactory->getObject('3.4'));
+		$converterChain->addConverter('4.0', $importConverterFactory->getObject('4.0'));
 
 		$adapter = new CImportDataAdapter(ZABBIX_EXPORT_VERSION, $converterChain);
 		$adapter->load($data);
