@@ -851,9 +851,13 @@ class CHost extends CHostGeneral {
 			}
 		}
 
-		if (isset($data['host'])) {
-			if (!preg_match('/^'.ZBX_PREG_HOST_FORMAT.'$/', $data['host'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect characters used for host name "%s".', $data['host']));
+		if (array_key_exists('host', $data)) {
+			$host_name_parser = new CHostNameParser();
+
+			if ($host_name_parser->parse($data['host']) != CParser::PARSE_SUCCESS) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Incorrect characters used for host name "%s".', $data['host'])
+				);
 			}
 
 			if (count($hosts) > 1) {
@@ -1442,7 +1446,9 @@ class CHost extends CHostGeneral {
 
 				$interfaces = zbx_toHash($interfaces, 'hostid');
 				foreach ($result as $hostid => $host) {
-					$result[$hostid]['interfaces'] = isset($interfaces[$hostid]) ? $interfaces[$hostid]['rowscount'] : 0;
+					$result[$hostid]['interfaces'] = array_key_exists($hostid, $interfaces)
+						? $interfaces[$hostid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1478,7 +1484,9 @@ class CHost extends CHostGeneral {
 				$screens = zbx_toHash($screens, 'hostid');
 
 				foreach ($result as $hostid => $host) {
-					$result[$hostid]['screens'] = isset($screens[$hostid]) ? $screens[$hostid]['rowscount'] : 0;
+					$result[$hostid]['screens'] = array_key_exists($hostid, $screens)
+						? $screens[$hostid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1660,6 +1668,8 @@ class CHost extends CHostGeneral {
 	 * @throws APIException if the input is invalid.
 	 */
 	protected function validateCreate(array $hosts) {
+		$host_name_parser = new CHostNameParser();
+
 		$host_db_fields = ['host' => null];
 
 		$groupids = [];
@@ -1678,7 +1688,7 @@ class CHost extends CHostGeneral {
 			}
 
 			// Validate "host" field.
-			if (!preg_match('/^'.ZBX_PREG_HOST_FORMAT.'$/', $host['host'])) {
+			if ($host_name_parser->parse($host['host']) != CParser::PARSE_SUCCESS) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
 					_s('Incorrect characters used for host name "%s".', $host['host'])
 				);
@@ -1899,6 +1909,8 @@ class CHost extends CHostGeneral {
 			'messageAllowedField' => _('Cannot update "%2$s" for a discovered host "%1$s".')
 		]);
 
+		$host_name_parser = new CHostNameParser();
+
 		$host_names = [];
 
 		foreach ($hosts as &$host) {
@@ -1939,7 +1951,7 @@ class CHost extends CHostGeneral {
 			}
 
 			if (array_key_exists('host', $host)) {
-				if (!preg_match('/^'.ZBX_PREG_HOST_FORMAT.'$/', $host['host'])) {
+				if ($host_name_parser->parse($host['host']) != CParser::PARSE_SUCCESS) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
 						_s('Incorrect characters used for host name "%s".', $host['host'])
 					);
