@@ -254,7 +254,6 @@ class CHttpTest extends CApiService {
 	 * @return array
 	 */
 	public function create($httptests) {
-		$httptests = $this->convertHttpPairs($httptests);
 		$this->validateCreate($httptests);
 
 		$httptests = Manager::HttpTest()->persist($httptests);
@@ -313,7 +312,7 @@ class CHttpTest extends CApiService {
 					'value' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('httpstep_field', 'value')]
 				]],
 				'follow_redirects' =>	['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF, HTTPTEST_STEP_FOLLOW_REDIRECTS_ON])],
-				'retrieve_mode' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS])],
+				'retrieve_mode' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS, HTTPTEST_STEP_RETRIEVE_MODE_BOTH])],
 				'timeout' =>			['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY | API_ALLOW_USER_MACRO, 'in' => '0:'.SEC_PER_HOUR],
 				'required' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httpstep', 'required')],
 				'status_codes' =>		['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httpstep', 'status_codes')]
@@ -343,7 +342,6 @@ class CHttpTest extends CApiService {
 	 * @return array
 	 */
 	public function update($httptests) {
-		$httptests = $this->convertHttpPairs($httptests);
 		$this->validateUpdate($httptests, $db_httptests);
 
 		Manager::HttpTest()->persist($httptests);
@@ -409,7 +407,7 @@ class CHttpTest extends CApiService {
 					'value' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('httpstep_field', 'value')]
 				]],
 				'follow_redirects' =>	['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF, HTTPTEST_STEP_FOLLOW_REDIRECTS_ON])],
-				'retrieve_mode' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS])],
+				'retrieve_mode' =>		['type' => API_INT32, 'in' => implode(',', [HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS, HTTPTEST_STEP_RETRIEVE_MODE_BOTH])],
 				'timeout' =>			['type' => API_TIME_UNIT, 'flags' => API_NOT_EMPTY | API_ALLOW_USER_MACRO, 'in' => '0:'.SEC_PER_HOUR],
 				'required' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httpstep', 'required')],
 				'status_codes' =>		['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('httpstep', 'status_codes')]
@@ -942,10 +940,8 @@ class CHttpTest extends CApiService {
 	 */
 	private function validateAuthParameters(array &$httptests, $method, array $db_httptests = null) {
 		foreach ($httptests as &$httptest) {
-			if (array_key_exists('authentication', $httptest)
-					|| array_key_exists('http_user', $httptest)
+			if (array_key_exists('authentication', $httptest) || array_key_exists('http_user', $httptest)
 					|| array_key_exists('http_password', $httptest)) {
-
 				$httptest += [
 					'authentication' => ($method === 'validateUpdate')
 						? $db_httptests[$httptest['httptestid']]['authentication']
@@ -963,21 +959,6 @@ class CHttpTest extends CApiService {
 						}
 					}
 				}
-				else {
-					foreach (['http_user', 'http_password'] as $field_name) {
-						$httptest += [
-							$field_name => ($method === 'validateUpdate')
-								? $db_httptests[$httptest['httptestid']][$field_name]
-								: ''
-						];
-
-						if ($httptest[$field_name] === '') {
-							self::exception(ZBX_API_ERROR_PARAMETERS,
-								_s('Incorrect value for field "%1$s": %2$s.', $field_name, _('cannot be empty'))
-							);
-						}
-					}
-				}
 			}
 		}
 		unset($httptest);
@@ -1055,23 +1036,21 @@ class CHttpTest extends CApiService {
 					else {
 						$db_httptest = $db_httptests[$httptest['httptestid']];
 						$db_httpstep = $db_httptest['steps'][$httpstep['httpstepid']];
-						$httpstep += ['retrieve_mode' => $db_httpstep['retrieve_mode']];
 						$httpstep += [
-							'posts' => ($httpstep['retrieve_mode'] == HTTPTEST_STEP_RETRIEVE_MODE_CONTENT)
+							'retrieve_mode' => $db_httpstep['retrieve_mode'],
+							'required' => $db_httpstep['required'],
+							'posts' => ($db_httpstep['retrieve_mode'] != HTTPTEST_STEP_RETRIEVE_MODE_HEADERS)
 								? $db_httpstep['posts']
-								: '',
-							'required' => ($httpstep['retrieve_mode'] == HTTPTEST_STEP_RETRIEVE_MODE_CONTENT)
-								? $db_httpstep['required']
 								: ''
 						];
 					}
 
 					if ($httpstep['retrieve_mode'] == HTTPTEST_STEP_RETRIEVE_MODE_HEADERS) {
-						if (($httpstep['posts'] !== '' && $httpstep['posts'] !== []) || $httpstep['required'] !== '') {
+						if ($httpstep['posts'] !== '' && $httpstep['posts'] !== []) {
 							$field_name = $httpstep['required'] !== '' ? 'required' : 'posts';
 
 							self::exception(ZBX_API_ERROR_PARAMETERS,
-								_s('Incorrect value for field "%1$s": %2$s.', $field_name, _('should be empty'))
+								_s('Incorrect value for field "%1$s": %2$s.', 'posts', _('should be empty'))
 							);
 						}
 					}
@@ -1081,72 +1060,4 @@ class CHttpTest extends CApiService {
 		}
 		unset($httptest);
 	}
-
-	/**
-	 * Convert string to HTTP pair array.
-	 *
-	 * @param string $data
-	 * @param string $delimiter
-	 *
-	 * @return mixed
-	 */
-	private function convertHTTPPairString($data, $delimiter) {
-		/* converts to pair array */
-		$pairs = array_values(array_filter(explode("\n", str_replace("\r", "\n", $data))));
-		foreach ($pairs as &$pair) {
-			$pair = explode($delimiter, $pair, 2);
-			$pair = [
-				'name' => $pair[0],
-				'value' => array_key_exists(1, $pair) ? $pair[1] : ''
-			];
-		}
-		unset($pair);
-
-		return $pairs;
-	}
-
-	/**
-	 * Convert headers and variables from string to HTTP pair array.
-	 * @deprecated conversion will be removed in future
-	 *
-	 * @param array  $httptests
-	 *
-	 * @return array
-	 */
-	private function convertHttpPairs($httptests) {
-		reset($httptests);
-
-		if (!is_int(key($httptests))) {
-			$httptests = [$httptests];
-		}
-
-		$fields = [
-			'headers' => ':',
-			'variables' => '='
-		];
-
-		foreach ($httptests as &$httptest) {
-			foreach ($fields as $field => $delimiter) {
-				if (is_array($httptest) && array_key_exists($field, $httptest) && is_string($httptest[$field])) {
-					$this->deprecated('using string format for field "'.$field.'" is deprecated.');
-					$httptest[$field] = $this->convertHTTPPairString($httptest[$field], $delimiter);
-				}
-			}
-
-			if (array_key_exists('steps', $httptest) && is_array($httptest['steps'])) {
-				foreach ($httptest['steps'] as &$step) {
-					foreach ($fields as $field => $delimiter) {
-						if (is_array($step) && array_key_exists($field, $step) && is_string($step[$field])) {
-							$this->deprecated('using string format for field "'.$field.'" is deprecated.');
-							$step[$field] = $this->convertHTTPPairString($step[$field], $delimiter);
-						}
-					}
-				}
-				unset($step);
-			}
-		}
-		unset($httptest);
-
-		return $httptests;
-	}
 }
