@@ -269,7 +269,7 @@ class DB {
 		$schema = self::getSchema($table_name);
 
 		if ($schema['fields'][$field_name]['type'] == self::FIELD_TYPE_TEXT) {
-			return ($DB['TYPE'] == ZBX_DB_DB2 || $DB['TYPE'] == ZBX_DB_ORACLE) ? 2048 : 65536;
+			return ($DB['TYPE'] == ZBX_DB_DB2 || $DB['TYPE'] == ZBX_DB_ORACLE) ? 2048 : 65535;
 		}
 
 		return $schema['fields'][$field_name]['length'];
@@ -385,8 +385,8 @@ class DB {
 							$length = mb_strlen($values[$field]);
 
 							if ($length > 2048) {
-								self::exception(self::SCHEMA_ERROR, _s('Value "%1$s" is too long for field "%2$s" - %3$d characters. Allowed length is 2048 characters.',
-									$values[$field], $field, $length));
+								self::exception(self::SCHEMA_ERROR, _s('Value "%1$s" is too long for field "%2$s" - %3$d characters. Allowed length is %4$d characters.',
+									$values[$field], $field, $length, 2048));
 							}
 						}
 						$values[$field] = zbx_dbstr($values[$field]);
@@ -1031,12 +1031,12 @@ class DB {
 
 			switch ($field_schema['type']) {
 				case self::FIELD_TYPE_ID:
-					$sql_parts['where'][] = dbConditionId($field_name, $options[$pk_option], false, true, false);
+					$sql_parts['where'][] = dbConditionId($field_name, $options[$pk_option]);
 					break;
 
 				case self::FIELD_TYPE_INT:
 				case self::FIELD_TYPE_UINT:
-					$sql_parts['where'][] = dbConditionInt($field_name, $options[$pk_option], false, true, false);
+					$sql_parts['where'][] = dbConditionInt($field_name, $options[$pk_option]);
 					break;
 
 				default:
@@ -1091,12 +1091,12 @@ class DB {
 
 			switch ($field_schema['type']) {
 				case self::FIELD_TYPE_ID:
-					$filter[] = dbConditionId(self::fieldId($field_name, $table_alias), $value, false, true, false);
+					$filter[] = dbConditionId(self::fieldId($field_name, $table_alias), $value);
 					break;
 
 				case self::FIELD_TYPE_INT:
 				case self::FIELD_TYPE_UINT:
-					$filter[] = dbConditionInt(self::fieldId($field_name, $table_alias), $value, false, true, false);
+					$filter[] = dbConditionInt(self::fieldId($field_name, $table_alias), $value);
 					break;
 
 				default:
