@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,32 +19,82 @@
 **/
 
 
+require_once dirname(__FILE__).'/js/configuration.template.list.js.php';
+
+$filter_tags = $data['filter']['tags'];
+if (!$filter_tags) {
+	$filter_tags = [['tag' => '', 'value' => '', 'operator' => TAG_OPERATOR_LIKE]];
+}
+
+$filter_tags_table = (new CTable())
+	->setId('filter-tags')
+	->addRow((new CCol(
+		(new CRadioButtonList('filter_evaltype', (int) $data['filter']['evaltype']))
+			->addValue(_('And/Or'), TAG_EVAL_TYPE_AND_OR)
+			->addValue(_('Or'), TAG_EVAL_TYPE_OR)
+			->setModern(true)
+		))->setColSpan(4)
+	);
+
+$i = 0;
+foreach ($filter_tags as $tag) {
+	$filter_tags_table->addRow([
+		(new CTextBox('filter_tags['.$i.'][tag]', $tag['tag']))
+			->setAttribute('placeholder', _('tag'))
+			->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH),
+		(new CRadioButtonList('filter_tags['.$i.'][operator]', (int) $tag['operator']))
+			->addValue(_('Contains'), TAG_OPERATOR_LIKE)
+			->addValue(_('Equals'), TAG_OPERATOR_EQUAL)
+			->setModern(true),
+		(new CTextBox('filter_tags['.$i.'][value]', $tag['value']))
+			->setAttribute('placeholder', _('value'))
+			->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH),
+		(new CCol(
+			(new CButton('filter_tags['.$i.'][remove]', _('Remove')))
+				->addClass(ZBX_STYLE_BTN_LINK)
+				->addClass('element-table-remove')
+		))->addClass(ZBX_STYLE_NOWRAP)
+	], 'form_row');
+
+	$i++;
+}
+$filter_tags_table->addRow(
+	(new CCol(
+		(new CButton('filter_tags_add', _('Add')))
+			->addClass(ZBX_STYLE_BTN_LINK)
+			->addClass('element-table-add')
+	))->setColSpan(3)
+);
+
 $filter = new CFilter(new CUrl('templates.php'));
-$filter->setProfile($data['profileIdx'])
+$filter
+	->setProfile($data['profileIdx'])
 	->setActiveTab($data['active_tab'])
 	->addFilterTab(_('Filter'), [
-		(new CFormList())->addRow(_('Name'),
-			(new CTextBox('filter_name', $data['filter']['name']))
-				->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
-				->setAttribute('autofocus', 'autofocus')
-		),
-		(new CFormList())->addRow(
-			(new CLabel(_('Linked templates'), 'filter_templates__ms')),
-			(new CMultiSelect([
-				'name' => 'filter_templates[]',
-				'object_name' => 'templates',
-				'data' => $data['filter']['templates'],
-				'popup' => [
-					'parameters' => [
-						'srctbl' => 'templates',
-						'srcfld1' => 'hostid',
-						'srcfld2' => 'host',
-						'dstfrm' => $filter->getName(),
-						'dstfld1' => 'filter_templates_'
+		(new CFormList())
+			->addRow(_('Name'),
+				(new CTextBox('filter_name', $data['filter']['name']))
+					->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
+					->setAttribute('autofocus', 'autofocus')
+			)
+			->addRow(
+				(new CLabel(_('Linked templates'), 'filter_templates__ms')),
+				(new CMultiSelect([
+					'name' => 'filter_templates[]',
+					'object_name' => 'templates',
+					'data' => $data['filter']['templates'],
+					'popup' => [
+						'parameters' => [
+							'srctbl' => 'templates',
+							'srcfld1' => 'hostid',
+							'srcfld2' => 'host',
+							'dstfrm' => $filter->getName(),
+							'dstfld1' => 'filter_templates_'
+						]
 					]
-				]
-			]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-		)
+				]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
+			),
+		(new CFormList())->addRow(_('Tags'), $filter_tags_table)
 	]);
 
 $widget = (new CWidget())
@@ -91,7 +141,8 @@ $table = (new CTableInfo())
 		_('Discovery'),
 		_('Web'),
 		_('Linked templates'),
-		_('Linked to')
+		_('Linked to'),
+		_('Tags')
 	]);
 
 foreach ($data['templates'] as $template) {
@@ -185,11 +236,19 @@ foreach ($data['templates'] as $template) {
 			CViewHelper::showNum($template['applications'])
 		],
 		[
-			new CLink(_('Items'), 'items.php?filter_set=1&hostid='.$template['templateid'].url_param('groupid')),
+			new CLink(_('Items'),
+				(new CUrl('items.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$template['templateid']])
+			),
 			CViewHelper::showNum($template['items'])
 		],
 		[
-			new CLink(_('Triggers'), 'triggers.php?hostid='.$template['templateid'].url_param('groupid')),
+			new CLink(_('Triggers'),
+				(new CUrl('triggers.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$template['templateid']])
+			),
 			CViewHelper::showNum($template['triggers'])
 		],
 		[
@@ -209,7 +268,8 @@ foreach ($data['templates'] as $template) {
 			CViewHelper::showNum($template['httpTests'])
 		],
 		$linkedTemplatesOutput,
-		$linkedToOutput
+		$linkedToOutput,
+		$data['tags'][$template['templateid']]
 	]);
 }
 
@@ -218,7 +278,16 @@ $form->addItem([
 	$data['paging'],
 	new CActionButtonList('action', 'templates',
 		[
-			'template.export' => ['name' => _('Export')],
+			'template.export' => ['name' => _('Export'), 'redirect' =>
+				(new CUrl('zabbix.php'))
+					->setArgument('action', 'export.templates.xml')
+					->setArgument('backurl', (new CUrl('templates.php'))
+						->setArgument('groupid', $data['pageFilter']->groupid)
+						->setArgument('page', getPageNumber())
+						->getUrl())
+					->getUrl()
+			],
+			'template.massupdateform' => ['name' => _('Mass update')],
 			'template.massdelete' => ['name' => _('Delete'), 'confirm' => _('Delete selected templates?')],
 			'template.massdeleteclear' => ['name' => _('Delete and clear'),
 				'confirm' => _('Delete and clear selected templates? (Warning: all linked hosts will be cleared!)')
